// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::ffi;
use glib::translate::*;

glib::wrapper! {
    pub struct Requisition(BoxedInline<ffi::GtkRequisition>);

    match fn {
        copy => |ptr| ffi::gtk_requisition_copy(ptr),
        free => |ptr| ffi::gtk_requisition_free(ptr),
        type_ => || ffi::gtk_requisition_get_type(),
    }
}

impl Requisition {
    #[doc(alias = "gtk_requisition_new")]
    pub fn new() -> Requisition {
        assert_initialized_main_thread!();
        unsafe { from_glib_full(ffi::gtk_requisition_new()) }
    }
}

impl Default for Requisition {
    fn default() -> Self {
        Self::new()
    }
}
